/*
 * Copyright (c) 2014, Olivier MATZ <zer0@droids-corp.org>
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <string.h>

#include <aversive.h>
#include <aversive/error.h>

#include <i2c.h>

#include "../fpv-common/i2c_commands.h"
#include "main.h"

void i2c_protocol_init(void)
{
}

/*** LED CONTROL ***/
void i2c_led_control(uint8_t l, uint8_t state)
{
	switch(l) {
	case 1:
		state? LED1_ON():LED1_OFF();
		break;
	case 2:
		state? LED2_ON():LED2_OFF();
		break;
	default:
		break;
	}
}

#ifdef notyet
static void i2c_test(uint16_t val)
{
	static uint16_t prev=0;

	if ( (val-prev) != 1 ) {
		WARNING(E_USER_I2C_PROTO, "Dupplicated msg %d", val);
	}
	prev = val;
	ext.nb_test_cmd ++;
}
#endif

static void i2c_send_status(void)
{
	struct i2c_ans_imuboard_status ans;
	static uint8_t x;

	i2c_flush();
	ans.hdr.cmd =  I2C_ANS_IMUBOARD_STATUS;

	/* status */
	ans.test = x++; /* XXX */
#if 0
	ans.mode = state_get_mode();
	ans.status = state_get_status();

	ans.left_cobroller_speed = imuboard.left_cobroller_speed;
	ans.right_cobroller_speed = imuboard.right_cobroller_speed;

	ans.cob_count = state_get_cob_count();
#endif

	i2c_send(I2C_ADD_MASTER, (uint8_t *) &ans,
		 sizeof(ans), I2C_CTRL_GENERIC);
}

void i2c_recvevent(uint8_t * buf, int8_t size)
{
	void *void_cmd = buf;

#if 0 /* XXX */
	static uint8_t a = 0;

	a++;
	if (a & 0x10)
		LED2_TOGGLE();

	if (size <= 0) {
		goto error;
	}
#endif

	switch (buf[0]) {

	/* Commands (no answer needed) */
	case I2C_CMD_GENERIC_LED_CONTROL:
		{
			struct i2c_cmd_led_control *cmd = void_cmd;
			if (size != sizeof (*cmd))
				goto error;
			i2c_led_control(cmd->led_num, cmd->state);
			break;
		}


	/* Add other commands here ...*/

	case I2C_REQ_IMUBOARD_STATUS:
		{
			//struct i2c_req_imuboard_status *cmd = void_cmd;
			if (size != sizeof (struct i2c_req_imuboard_status))
				goto error;

			i2c_send_status();
			break;
		}

	default:
		goto error;
	}

 error:
	/* log error on a led ? */
	return;
}

void i2c_recvbyteevent(__attribute__((unused)) uint8_t hwstatus,
		       __attribute__((unused)) uint8_t i, 
		       __attribute__((unused)) int8_t c)
{
}

void i2c_sendevent(__attribute__((unused)) int8_t size)
{
}


