#include <stdio.h>
#include <string.h>
#include <math.h>

#include <aversive/irq_lock.h>
#include <aversive/wait.h>
#include <aversive/pgmspace.h>

#include <uart.h>

#include "i2c_helper.h"
#include "mpu6050.h"
#include "vector.h"
#include "matrix.h"
#include "MadgwickAHRS.h"
#include "sd_log.h"
#include "imu.h"
#include "main.h"

static struct callout imu_timer;

double roll;
double pitch;
double yaw;

void quaternion2euler(void)
{
	roll = atan2f(2.0f * (q0 * q1 + q2 * q3), q0*q0 - q1*q1 - q2*q2 + q3*q3);
	pitch = -asinf(2.0f * (q1 * q3 - q0 * q2));
	yaw = atan2f(2.0f * (q1 * q2 + q0 * q3), q0*q0 + q1*q1 - q2*q2 - q3*q3);
}

static void imu_cb(struct callout_mgr *cm, struct callout *tim, void *arg)
{
	int16_t mpu6050_axes[10];

	(void)arg;

	mpu6050_read_all_axes(mpu6050_axes);

	MadgwickAHRSupdate(mpu6050_gx,
		mpu6050_gy,
		mpu6050_gz,
		mpu6050_ax,
		mpu6050_ay,
		mpu6050_az,
		mpu6050_mx,
		mpu6050_my,
		mpu6050_mz
	);

	quaternion2euler();

	callout_schedule(cm, tim, 2);
}

void imu_init(void)
{
	mpu6050_init();

	callout_init(&imu_timer, imu_cb, NULL, IMU_PRIO);
	callout_schedule(&imuboard.intr_cm, &imu_timer, 20); /* every 20ms */
}

void imu_get_info(struct imu_info *imu)
{
	imu->gx = mpu6050_gx;
	imu->gy = mpu6050_gy;
	imu->gz = mpu6050_gz;
	imu->ax = mpu6050_ax;
	imu->ay = mpu6050_ay;
	imu->az = mpu6050_az;
	imu->mx = mpu6050_mx;
	imu->my = mpu6050_my;
	imu->mz = mpu6050_mz;
	imu->temp = 0;
}


int imu_loop(void)
{
	char buf[128];
	int16_t len;
	uint32_t ms;
	uint8_t flags;
	struct imu_info imu;

	while (1) {

		IRQ_LOCK(flags);
		ms = global_ms;
		imu_get_info(&imu);
		IRQ_UNLOCK(flags);

		if (sd_log_enabled()) {
			len = snprintf(buf, sizeof(buf),
				"%"PRIu32"\t"
				"gyro %+3.3f\t%+3.3f\t%+3.3f\t\t"
				"accel %+3.3f\t%+3.3f\t%+3.3f\t\t"
				"magnet %+3.3f\t%+3.3f\t%+3.3f\r\n",
				ms,
				imu.gx, imu.gy, imu.gz,
				imu.ax, imu.ay, imu.az,
				imu.mx, imu.my, imu.mz);
			if (sd_log_write(buf, len) != len) {
				printf_P(PSTR("error writing to file\n"));
				return -1;
			}
		}

	}

	return 0;
}
